{
    Model-GUI-Mediator demo

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    Description:
      This unit is part of the Model-GUI-Mediator article written for
      the Toolbox Magazine. To compile this project you need FPC 2.2.0
      or later and the fpGUI Toolkit (http://fpgui.sourceforge.net/).
}

unit frmContactEdit;

{$mode objfpc}{$H+}

interface

uses
  SysUtils,
  Classes,
  fpg_base,
  fpg_main,
  fpg_edit,
  fpg_widget,
  fpg_form,
  fpg_label,
  fpg_button,
  fpg_dialogs,
  fpg_panel,
  fpg_memo,
  fpg_popupcalendar,
  model,
  views;

type

  TContactEditForm = class(TfpgForm)
  private
    {@VFD_HEAD_BEGIN: ContactEditForm}
    lblName: TfpgLabel;
    edtName: TfpgEdit;
    lblAddress: TfpgLabel;
    lblDOB: TfpgLabel;
    btnOK: TfpgButton;
    btnCancel: TfpgButton;
    pnlName: TfpgPanel;
    meAddress: TfpgMemo;
    edDOB: TfpgCalendarCombo;
    {@VFD_HEAD_END: ContactEditForm}
    FData: TContact;
    FmedName: TContactNameEditView;
    FmedName2: TContactNamePanelView;
    FmedAddress: TContactAddressMemoView;
    FmedDOB: TContactDateOfBirthEditView;
    procedure FormShow(Sender: TObject);
    procedure CreateMediators;
    procedure SetupCaptions;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AfterCreate; override;
    property Data: TContact read FData;
  end;

{@VFD_NEWFORM_DECL}

function EditContact(AContact: TContact): Boolean;


implementation

uses
  Constants,
  fpg_constants,
  fpg_utils;

function EditContact(AContact: TContact): Boolean;
var
  frm: TContactEditForm;
begin
  frm := TContactEditForm.Create(nil);
  try
    frm.FData := AContact;
    Result := frm.ShowModal = mrOK;
  finally
    frm.Free;
  end;
end;

{@VFD_NEWFORM_IMPL}

procedure TContactEditForm.FormShow(Sender: TObject);
begin
  CreateMediators;
  { Now all observers will have a change to update
    themselves }
  Data.NotifyObservers;
end;

procedure TContactEditForm.CreateMediators;
begin
  { Create the mediating view instances }
  FmedName    :=
      TContactNameEditView.CreateCustom(edtName, Data);
  FmedName2   :=
      TContactNamePanelView.CreateCustom(pnlName, Data);
  FmedAddress :=
      TContactAddressMemoView.CreateCustom(meAddress, Data);
  FmedDOB     :=
      TContactDateOfBirthEditView.CreateCustom(edDOB, Data);
end;

procedure TContactEditForm.SetupCaptions;
begin
  WindowTitle     := cEditContact;
  btnOK.Text      := rsOK;
  btnCancel.Text  := rsCancel;
  lblName.Text    := fpgAddColon(rsName);
  lblAddress.Text := fpgAddColon(cAddress);
  lblDOB.Text     := fpgAddColon(cDOB);
end;

constructor TContactEditForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OnShow := @FormShow;
end;

destructor TContactEditForm.Destroy;
begin
  FmedName.Free;
  FmedName2.Free;
  FmedAddress.Free;
  FmedDOB.Free;
  inherited Destroy;
end;

procedure TContactEditForm.AfterCreate;
begin
  {@VFD_BODY_BEGIN: ContactEditForm}
  Name := 'ContactEditForm';
  SetPosition(357, 385, 302, 267);
  WindowTitle := 'Contact Maintenance';

  lblName := TfpgLabel.Create(self);
  with lblName do
  begin
    Name := 'lblName';
    SetPosition(8, 40, 192, 16);
    FontDesc := '#Label1';
    Hint := '';
    Text := 'lblName';
  end;

  edtName := TfpgEdit.Create(self);
  with edtName do
  begin
    Name := 'edtName';
    SetPosition(8, 56, 192, 22);
    TabOrder := 1;
    Text := '';
    FontDesc := '#Edit1';
  end;

  lblAddress := TfpgLabel.Create(self);
  with lblAddress do
  begin
    Name := 'lblAddress';
    SetPosition(8, 84, 192, 16);
    FontDesc := '#Label1';
    Hint := '';
    Text := 'lblAddress';
  end;

  lblDOB := TfpgLabel.Create(self);
  with lblDOB do
  begin
    Name := 'lblDOB';
    SetPosition(8, 164, 192, 16);
    FontDesc := '#Label1';
    Hint := '';
    Text := 'lblDOB';
  end;

  btnOK := TfpgButton.Create(self);
  with btnOK do
  begin
    Name := 'btnOK';
    SetPosition(128, 233, 80, 24);
    Anchors := [anRight,anBottom];
    Text := 'OK';
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    ModalResult := mrOK;
    TabOrder := 4;
  end;

  btnCancel := TfpgButton.Create(self);
  with btnCancel do
  begin
    Name := 'btnCancel';
    SetPosition(212, 233, 80, 24);
    Anchors := [anRight,anBottom];
    Text := 'Cancel';
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    ModalResult := mrCancel;
    TabOrder := 5;
  end;

  pnlName := TfpgPanel.Create(self);
  with pnlName do
  begin
    Name := 'pnlName';
    SetPosition(8, 8, 285, 24);
    Anchors := [anLeft,anRight,anTop];
    Style := bsLowered;
    Text := 'pnlName';
  end;

  meAddress := TfpgMemo.Create(self);
  with meAddress do
  begin
    Name := 'meAddress';
    SetPosition(8, 100, 192, 60);
    FontDesc := '#Edit1';
    TabOrder := 2;
  end;

  edDOB := TfpgCalendarCombo.Create(self);
  with edDOB do
  begin
    Name := 'edDOB';
    SetPosition(8, 180, 136, 22);
    FontDesc := '#List';
    TabOrder := 3;
    DateFormat := 'yyyy-mm-dd';
  end;

  {@VFD_BODY_END: ContactEditForm}

  { Setup captions, which handle translations }
  SetupCaptions
end;


end.
