{
    Model-GUI-Mediator demo

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    Description:
      This unit is part of the Model-GUI-Mediator article written for
      the Toolbox Magazine. To compile this project you need FPC 2.2.0
      or later and the fpGUI Toolkit (http://fpgui.sourceforge.net/).
}

unit frmMain;

{$mode objfpc}{$H+}

interface

uses
  SysUtils,
  Classes,
  fpg_base,
  fpg_main,
  fpg_widget,
  fpg_form,
  fpg_button,
  fpg_listbox,
  fpg_dialogs,
  fpg_menu,
  model;

type

  TMainForm = class(TfpgForm)
  private
    {@VFD_HEAD_BEGIN: MainForm}
    mnuFile: TfpgPopupMenu;
    mnuEdit: TfpgPopupMenu;
    MainMenu: TfpgMenuBar;
    LB: TfpgListBox;
    btnAdd: TfpgButton;
    btnEdit: TfpgButton;
    btnDelete: TfpgButton;
    {@VFD_HEAD_END: MainForm}
    FContactList: TContactList;
    procedure miFileExitClicked(Sender: TObject);
    procedure acAddContact(Sender: TObject);
    procedure acEditContact(Sender: TObject);
    procedure acDeleteContact(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PopulateListBox;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AfterCreate; override;
  end;

{@VFD_NEWFORM_DECL}

implementation

uses
  frmContactEdit, fpg_constants, constants;

{@VFD_NEWFORM_IMPL}

procedure TMainForm.miFileExitClicked(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.acAddContact(Sender: TObject);
var
  lData: TContact;
begin
  lData := TContact.Create;
  if EditContact(lData) then
  begin
    FContactList.Add(lData);
    PopulateListbox;
  end
  else
    lData.Free;
end;

procedure TMainForm.acEditContact(Sender: TObject);
var
  lData: TContact;
  i: integer;
begin
  i := LB.FocusItem;  // save focus item
  if i < 0 then
    Exit;
  lData := TContact(LB.Items.Objects[i]);
  if EditContact(lData) then
  begin
    PopulateListbox;
    LB.FocusItem := i; // restore focus item
  end;
end;

procedure TMainForm.acDeleteContact(Sender: TObject);
var
  lData: TContact;
begin
  if LB.FocusItem < 0 then
    Exit;
  lData := TContact(LB.Items.Objects[LB.FocusItem]);
  FContactList.Remove(lData);
  PopulateListbox;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  FContactList := CreateContacts;
  PopulateListbox;
end;

procedure TMainForm.PopulateListBox;
var
  lData: TContact;
  i: integer;
begin
  LB.Items.Clear;
  for i := 0 to FContactList.Count-1 do
  begin
    lData := FContactList.Items[i];
    LB.Items.AddObject(lData.Name, lData);
  end;
end;

constructor TMainForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OnShow := @FormShow;
end;

destructor TMainForm.Destroy;
begin
  FContactList.Free;
  inherited Destroy;
end;

procedure TMainForm.AfterCreate;
begin
  {@VFD_BODY_BEGIN: MainForm}
  Name := 'MainForm';
  SetPosition(358, 396, 308, 280);
  WindowTitle := 'Model-GUI-Mediator demo';

  mnuFile := TfpgPopupMenu.Create(self);
  with mnuFile do
  begin
    Name := 'mnuFile';
    SetPosition(0, 0, 120, 32);
    AddMenuItem(rsExit, '', @miFileExitClicked);
  end;

  mnuEdit := TfpgPopupMenu.Create(self);
  with mnuEdit do
  begin
    Name := 'mnuEdit';
    SetPosition(0, 0, 120, 32);
    AddMenuItem(rsInsert, '', @acAddContact);
    AddMenuItem(rsEdit, '', @acEditContact);
    AddMenuItem(rsDelete, '', @acDeleteContact);
  end;

  MainMenu := TfpgMenuBar.Create(self);
  with MainMenu do
  begin
    Name := 'MainMenu';
    SetPosition(0, 0, 308, 24);
    Anchors := [anLeft,anRight,anTop];
    AddMenuItem(cFile, nil).SubMenu := mnuFile;
    AddMenuItem(rsEdit, nil).SubMenu := mnuEdit;
  end;

  LB := TfpgListBox.Create(self);
  with LB do
  begin
    Name := 'LB';
    SetPosition(8, 68, 292, 196);
    Anchors := [anLeft,anRight,anTop,anBottom];
    FontDesc := '#List';
    TabOrder := 3;
  end;

  btnAdd := TfpgButton.Create(self);
  with btnAdd do
  begin
    Name := 'btnAdd';
    SetPosition(8, 40, 80, 24);
    Text := rsInsert;
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 1;
    OnClick := @acAddContact;
  end;

  btnEdit := TfpgButton.Create(self);
  with btnEdit do
  begin
    Name := 'btnEdit';
    SetPosition(92, 40, 80, 24);
    Text := rsEdit;
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 2;
    OnClick := @acEditContact;
  end;

  btnDelete := TfpgButton.Create(self);
  with btnDelete do
  begin
    Name := 'btnDelete';
    SetPosition(176, 40, 80, 24);
    Text := rsDelete;
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 3;
    OnClick := @acDeleteContact;
  end;

  {@VFD_BODY_END: MainForm}
end;


end.
