{
    Model-GUI-Mediator demo

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    Description:
      This unit is part of the Model-GUI-Mediator article written for
      the Toolbox Magazine. To compile this project you need FPC 2.2.0
      or later and the fpGUI Toolkit (http://fpgui.sourceforge.net/).
}

unit mediators;

{$mode objfpc}{$H+}

interface

uses
  observer, fpg_edit, fpg_panel, fpg_memo, fpg_popupcalendar;

type
  TBaseMediator = class(TObserver)
  protected
    { Subscribe to the Subject }
    procedure DoAttachment(ASubject: TSubject);
    { placeholder method where we update the
      Subject due to GUI changes }
    procedure DoGuiToObject; virtual;
    { placedolder method where we update the GUI
      due to Subject changes }
    procedure DoObjectToGui; virtual;
  public
    procedure Update(ASubject: TSubject); override;
  end;


  { Mediator handling TfpgEdit GUI component }
  TEditMediator = class(TBaseMediator)
  private
    FGUIControl: TfpgEdit;
    { event handler to detect changes in the GUI component }
    procedure InternalOnChange(Sender: TObject);
  public
    { A custom constructor so we can pass in both the GUI
      control and the Subject we want to observe. }
    constructor CreateCustom(AControl: TfpgEdit;
        ASubject: TSubject);
    { A read-only property pointing to the GUI Control
      we are managing. }
    property GUIControl: TfpgEdit read FGUIControl;
  end;


  { Mediator handling TfpgPanel GUI component }
  TPanelMediator = class(TBaseMediator)
  private
    FGUIControl: TfpgPanel;
  public
    constructor CreateCustom(AControl: TfpgPanel;
        ASubject: TSubject);
    property GUIControl: TfpgPanel read FGUIControl;
  end;


  { Mediator handling TfpgMemo GUI component }
  TMemoMediator = class(TBaseMediator)
  private
    FGUIControl: TfpgMemo;
    procedure InternalOnExit(Sender: TObject);
  public
    constructor CreateCustom(AControl: TfpgMemo;
        ASubject: TSubject);
    property GUIControl: TfpgMemo read FGUIControl;
  end;


  { Mediator handling TfpgCalendarCombo GUI component }
  TCalendarMediator = class(TBaseMediator)
  private
    FGUIControl: TfpgCalendarCombo;
    procedure InternalOnChange(Sender: TObject);
  public
    constructor CreateCustom(AControl: TfpgCalendarCombo;
        ASubject: TSubject);
    property GUIControl: TfpgCalendarCombo read FGUIControl;
  end;


implementation

uses
  SysUtils;


{ TBaseMediator }

procedure TBaseMediator.DoAttachment(ASubject: TSubject);
begin
  if ASubject = nil then
    raise Exception.Create('You must specify a Subject' +
      ' to be observed.');
  { set internal subject reference for later use }
  FSubject := ASubject;
  { Register as an observer of Subject }
  FSubject.AttachObserver(self);
end;

procedure TBaseMediator.DoGuiToObject;
begin
  // Do nothing yet.
end;

procedure TBaseMediator.DoObjectToGui;
begin
  // Do nothing yet.
end;

procedure TBaseMediator.Update(ASubject: TSubject);
begin
  DoObjectToGui;
end;


{ TEditMediator }

procedure TEditMediator.InternalOnChange(Sender: TObject);
begin
  DoGuiToObject;
end;

constructor TEditMediator.CreateCustom(
    AControl: TfpgEdit; ASubject: TSubject);
begin
  Create;
  FGUIControl := AControl;
  FGUIControl.OnChange := @InternalOnChange;
  DoAttachment(ASubject);
end;


{ TPanelMediator }

constructor TPanelMediator.CreateCustom(
    AControl: TfpgPanel; ASubject: TSubject);
begin
  Create;
  FGUIControl := AControl;
  DoAttachment(ASubject);
end;


{ TMemoMediator }

procedure TMemoMediator.InternalOnExit(Sender: TObject);
begin
  DoGuiToObject;
end;

constructor TMemoMediator.CreateCustom(
    AControl: TfpgMemo; ASubject: TSubject);
begin
  Create;
  FGUIControl := AControl;
  FGUIControl.OnExit := @InternalOnExit;
  DoAttachment(ASubject);
end;


{ TCalendarMediator }

procedure TCalendarMediator.InternalOnChange(Sender: TObject);
begin
  DoGuiToObject;
end;

constructor TCalendarMediator.CreateCustom(
    AControl: TfpgCalendarCombo; ASubject: TSubject);
begin
  Create;
  FGUIControl := AControl;
  FGUIControl.OnChange := @InternalOnChange;
  DoAttachment(ASubject);
end;

end.

