{
    Model-GUI-Mediator demo

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    Description:
      This unit is part of the Model-GUI-Mediator article written for
      the Toolbox Magazine. To compile this project you need FPC 2.2.0
      or later and the fpGUI Toolkit (http://fpgui.sourceforge.net/).
}

unit model;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, observer, contnrs;

type
  { our business object we are going to observe }
  TContact = class(TSubject)
  private
    FAddress: string;
    FDateOfBirth: TDateTime;
    FName: string;
    procedure SetAddress(const AValue: string);
    procedure SetDateOfBirth(const AValue: TDateTime);
    procedure SetName(const AValue: string);
  public
    constructor Create; override;
    property Name: string
              read FName write SetName;
    property Address: string
              read FAddress write SetAddress;
    property DateOfBirth: TDateTime
              read FDateOfBirth write SetDateOfBirth;
  end;


  TContactList = class(TSubject)
  private
    FList: TObjectList;
    function GetCount: integer;
    function GetItem(Index: Integer): TContact;
    procedure SetItem(Index: Integer; const AValue: TContact);
  public
    constructor Create; override;
    destructor Destroy; override;
    function Add(AContact: TContact): integer;
    function Remove(AContact: TContact): integer;
    property Count: integer read GetCount;
    property Items[Index: Integer]: TContact read GetItem write SetItem; default;
  end;


{ Helper function to create some sample data. }
function CreateContacts: TContactList;


implementation


function CreateContacts: TContactList;
var
  lData: TContact;
begin
  Result := TContactList.Create;

  lData := TContact.Create;
  lData.Name := 'Graeme Geldenhuys';
  lData.Address := '21b Stellenberg Road';
  lData.DateOfBirth := EncodeDate(1974, 12, 16);
  Result.Add(lData);

  lData := TContact.Create;
  lData.Name := 'Martin Lee';
  lData.Address := 'High Street';
  lData.DateOfBirth := EncodeDate(1965, 2, 25);
  Result.Add(lData);

  lData := TContact.Create;
  lData.Name := 'Angela Hobkirk';
  lData.Address := 'Beach Avenue';
  lData.DateOfBirth := EncodeDate(1980, 5, 17);
  Result.Add(lData);

  lData := TContact.Create;
  lData.Name := 'John Guthrie';
  lData.Address := 'Irene Avenue';
  lData.DateOfBirth := EncodeDate(1976, 7, 8);
  Result.Add(lData);
end;


{ TContact }

procedure TContact.SetName(const AValue: string);
begin
  if FName=AValue then exit;
  FName:=AValue;
  NotifyObservers;
end;

constructor TContact.Create;
begin
  inherited Create;
  FDateOfBirth := Trunc(Now);
end;

procedure TContact.SetAddress(const AValue: string);
begin
  if FAddress=AValue then exit;
  FAddress:=AValue;
  NotifyObservers;
end;

procedure TContact.SetDateOfBirth(const AValue: TDateTime);
begin
  if FDateOfBirth = AValue then
    Exit; //==>
  FDateOfBirth := AValue;
  NotifyObservers;
end;

{ TContactList }

function TContactList.GetCount: integer;
begin
  Result := FList.Count
end;

function TContactList.GetItem(Index: Integer): TContact;
begin
  Result := TContact(FList.Items[Index]);
end;

procedure TContactList.SetItem(Index: Integer; const AValue: TContact);
begin
  FList.Items[Index] := AValue;
end;

constructor TContactList.Create;
begin
  inherited Create;
  FList := TObjectList.Create(True);
end;

destructor TContactList.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

function TContactList.Add(AContact: TContact): integer;
begin
  Result := FList.Add(AContact);
  NotifyObservers;
end;

function TContactList.Remove(AContact: TContact): integer;
begin
  Result := FList.Remove(AContact);
  NotifyObservers;
end;


end.

